/*
 * @copyright        © Copyright 2006-2012, epages GmbH, All Rights Reserved.
 *
 * @module            de_epages.presentation.inc.dndsortabletable
 */

/*jslint browser: true, regexp: true*/
/*global define*/

 define("de_epages/presentation/inc/dndsortabletable",[
    "jquery",

    "jquery/ui/sortable",
    "$ready!"
], function ($) {

    return function (options) {
        if (typeof options !== 'object') {
            console.warn('dndSortableTable: No options. Initialization aborted.');
            return;
        }

        var defaults = {
                changeAction: 'Dummy',
                objectId: 0,
                multipage: false,
                multiselect: false,
                showElements: [],
                helperString: '<div style="position: absolute;"><table style="width: 100%;"><tbody class="ep-dndsorter-helperrowwrapper"></tbody></table></div>',
                selectors : {
                    tableBody: '.ContentList > tbody',
                    sortColumnHead: '.ContentList > thead > tr > td:last-child, .ContentList > thead > tr > th:last-child',
                    sortColumnBody: '.ContentList > tbody > tr > td:last-child:has(input[name*="Position"])',
                    inputField: '.InputField > input[name=ListPosition]',
                    busyTarget: 'body'
                }
            },
            o = $.extend(true, defaults, options),
            dom = {
                busyTarget: $(o.selectors.busyTarget),
                tableBody: $(o.selectors.tableBody),
                sortColumnHead: $(o.selectors.sortColumnHead),
                sortColumnBody: $(o.selectors.sortColumnBody),
                inputs: $(o.selectors.tableBody).find(o.selectors.inputField)
            },
            startVal = parseInt(dom.inputs.eq(0).val()),

            originalOlderSibling,
            originalRadioButtons = [];

        //test required elements, exit if not provided
        if (['tableBody', 'sortColumnHead', 'sortColumnBody', 'inputs'].filter(function(item) {
                return dom[item].length;
            }).length !== 4) {
            console.warn('dndSortableTable: Missing selectors. Initialization aborted.');
            return;
        }

        // if no target for busy, fallback to body
        if (!dom.busyTarget.length) {
            dom.busyTarget = $('body');
        }

        dom.form = dom.tableBody.closest('form');

        // append style class and icons to column with position input
        dom.sortColumnHead.addClass('ep-sortabletable-position-col');

        dom.sortColumnBody
            .append($('<span class="ep-sortable-handle CursorMove"><i class="fa fa-lg fa-ellipsis-v"/><i class="fa fa-lg fa-ellipsis-v"/></span>'))
            .addClass('ep-sortabletable-position-col');

        // init sortable
        dom.tableBody.sortable({
            tolerance: 'cursor',
            items: "> tr:not(.NewEntry):has(" + o.selectors.inputField + ")",
            revert: 200,
            helper: function () {
                return $(o.helperString);
            },
            cursorAt: { top: 31 },
            forceHelperSize: true,
            start: function(event, ui) {
                var tableRowSet,
                    oldRowSet;

                originalPosition = dom.tableBody.children().index(ui.item);

                // radio buttons when copied, will remove the checked flag from the original buttons
                // when the copied items are removed from the DOM, the originals lose their checked status
                // so back these up and restore them later
                originalRadioButtons = [];
                ui.item.find('input[type=radio]:checked').each(function(){
                    originalRadioButtons.push({
                      'name' : this.name,
                      'value': this.value
                    });
                });

                // remove class RowSelected both from clicked row and placeholder, as not to affect them lateron
                if (ui.item.hasClass('RowSelected')) {
                    ui.item.removeClass('RowSelected');
                    ui.item.addClass('RowSelected_old');
                }
                $('.ui-sortable-placeholder').removeClass('RowSelected');
                // hide other selected rows
                if (o.multiselect) {
                    $('.RowSelected').css('display', 'none');
                }
                dom.tableBody.addClass('sorting');
                // append a clone to tbody inside helper
                ui.helper.find('tbody').append(ui.item.clone().show());

                // create a deep clone to prevent table from flickering
                ui.placeholder.html(ui.item.html());

                // get td elements and enforce width
                tableRowSet = ui.helper.find('tbody.ep-dndsorter-helperrowwrapper > tr > td');
                oldRowSet = ui.placeholder.find('> td');

                oldRowSet.each(function(index) {
                    tableRowSet.eq(index).css('width', oldRowSet.eq(index).outerWidth() + 'px');
                });

                // if no showElements show all, otherwise hide all and only show specified td's
                if (o.showElements.length) {
                    tableRowSet.addClass('Opacity0');
                    o.showElements.forEach(function(number) {
                        tableRowSet.eq(number).removeClass('Opacity0');
                    });
                }

            },
            sort: function(event, ui) {
                var tableRect = dom.tableBody[0].parentElement.getBoundingClientRect();

                if (!(tableRect.left < event.clientX && event.clientX < tableRect.left + tableRect.width
                    && tableRect.top < event.clientY && event.clientY < tableRect.top + tableRect.height)
                    && dom.tableBody.children().index(ui.placeholder) !== originalPosition) {
                    if (originalPosition === 0) {
                        dom.tableBody.prepend(ui.placeholder);
                    } else {
                        dom.tableBody.children().eq(originalPosition).after(ui.placeholder);
                    }
                }
            },

            stop: function(event, ui) {
                var tableRect = dom.tableBody[0].parentElement.getBoundingClientRect(),
                    $selectedRows = $('.RowSelected'),
                    numberOfRows =  $selectedRows.length + 1,
                    $clickedRowInput = ui.item.find(o.selectors.inputField),
                    clickedItemVal = parseInt($clickedRowInput.val()),
                    parentLink = ui.item.closest('a'),

                    rowsStartVal,
                    $input,
                    otherItemVal,
                    formData,
                    placeholder,

                    // sort items for multiselect, fraction positions in case selected rows > 10;
                    sortItems = function (startVal) {
                        // calculate position for main item & trigger change
                        $clickedRowInput.val(((startVal > 0) ? startVal : 0) + 0.00001);
                        $clickedRowInput.trigger('change');

                        // calculate position for other items & trigger change
                        if (o.multiselect) {
                            $selectedRows.each(function(index, elem) {
                                $input = $(elem).find(o.selectors.inputField);
                                $input.val(((startVal > 0) ? startVal : 0) + ((index + 1) * ((parseInt($input.val()) > clickedItemVal)? 0.0001 : 0.00000001)));
                                $input.trigger('change');
                            });
                        }
                    };

                // undo temp selected class
                if (ui.item.hasClass('RowSelected_old')) {
                    ui.item.removeClass('RowSelected_old');
                    ui.item.addClass('RowSelected');
                }

                originalRadioButtons.forEach(function(info){
                    $(o.selectors.tableBody + ' input[type=radio][name=' + info.name + '][value=' + info.value + ']').prop('checked', true);
                });

                // check if mouse over table body or if one of the pager items is hovered
                if (!(tableRect.left < event.clientX && event.clientX < tableRect.left + tableRect.width
                        && tableRect.top < event.clientY && event.clientY < tableRect.top + tableRect.height)) {
                    // cancel sort action
                    dom.tableBody.sortable('cancel');
                } else {

                    if (!ui.item.parent(o.selectors.tableBody).length) {
                        dom.tableBody.append(ui.item);
                    }

                    // add busy screen
                    dom.busyTarget.busy('show');

                    // place other rows around main sort item
                    if (o.multiselect) {
                        $selectedRows.each(function(index, elem) {
                            otherItemVal = parseInt($(elem).find(o.selectors.inputField).val());
                            if (otherItemVal < clickedItemVal) {
                                ui.item.before(elem);

                            } else if (otherItemVal > clickedItemVal) {
                                ui.item.after(elem);
                            }
                        });
                    }

                    // recalculate all rows & trigger change
                    dom.tableBody.children('tr:not(.NewEntry)').each(function(index, elem) {
                        $input = $(elem).find(o.selectors.inputField);
                        $input.val(index * 10 + startVal);
                    });
                    formData = dom.form.serializeArray();

                    formData = formData.filter(function(item) {
                        return !(item.name === "ChangeAction" || item.name ==="ObjectID");
                    });

                    formData.push({name: 'ChangeAction', value: o.changeAction}, {name: 'ObjectID', value: o.objectId});

                    // submit form per ajax, serialize form data to array of objects, transform Array of objects to Object with name-value-pairs
                    ep.ajax({
                        url:  '?',
                        type: 'POST',
                        data: formData,
                    }).done(function() {

                        //hide busy
                        dom.busyTarget.busy('hide');
                    }).fail(function() {

                        // on fail try a regular submit also to display possible errors
                        dom.form.submit();
                    });
                }

                // undo sorting style
                if (o.multiselect) {
                    $selectedRows.css('display', 'table-row');
                }
                dom.tableBody.removeClass('sorting');
            }
        });
    };
});
