/**
 * Create Variation Wizard handling on top of the existing html structure
 *
 * The `.productUiVariationwizardUnity()` method adds functionality easily create product variations.
 *
 *
 * ### Examples
 * Apply .productUiVariationwizardUnity() to Button element.
 *
 * JavaScript:
 *
 *      require([
 *        'jquery',
 *        'de_epages/product/ui/variationwizard-unity',
 *
 *        '$ready!'
 *      ], function ($, ep) {
 *
 *        $('form').productUiVariationwizardUnity({
 *            productTypeSelector: '#' + 'ClassID',
 *            productTypeAliasInput: '#' + 'Alias',
 *            productTypeAliasInputWrapper: '#' + 'NewProductTypeNameInput',
 *
 *            attributeArea:  '#' + 'ep-step2-wrapper',
 *            attributeAreaHeadline: '#' + 'AttributeContainerAttributeHeadline',
 *
 *            attributes: '.ep-variationassistant-ownattributes',
 *            attributeAlias: '.ep-variationassistant-attributealias',
 *            attributeStep: '#' + 'EnterAttributes',
 *            attributeContainer: '.ep-variationassistant-attributecontainer',
 *            attributeCount: #Counter,
 *            attributeValues: '.ep-variationassistant-details',
 *            deleteAttribute: '.ep-variationassistant-deleteAttribute',
 *            isUsed: '.ep-variationassistant-isusedcheckbox > input[type="checkbox"]',
 *            newAttributeButton: '#' + 'EnterNewAttributeButton',
 *
 *            defaultList: $('#' + 'DefaultList'),
 *            selectDefaultContainer: $('#' + 'SelectDefaultContainer'),
 *
 *            productTable: $('#' + 'ProductTable'),
 *            productListContainer: $('#' + 'ProductListContainer'),
 *
 *            backButton: '#ep-variationassistant-backButton',
 *            nextButton: '#ep-variationassistant-nextButton',
 *            finishButton: '#ep-variationassistant-finishButton',
 *
 *            noAttributesError: '#NoAttributes',
 *
 *            existingProducts: existingProducts,
 *            bus: $Form,
 *            baseAlias: '#Alias',
 *            baseName: '#Name',
 *
 *            maxVariations: #Shop.FeatureMaxValue.ProductMaxSubProducts,
 *            maxAttributes: #Shop.FeatureMaxValue.Variations,
 *            maxAttributeValues: #Shop.FeatureMaxValue.VariationAttributeValues ,
 *            currentClass: #Class.ID,
 *            basicUrl: '?ObjectID=#ID&ChangeAction=SaveProductType&ViewAction=UnityMBO-ViewProductVariationWizard'
 *          });
 *
 *      });
 *
 * HTML:
 *
 *    <!-- EnterAttributesArea: start -->
 *      <div id="EnterAttributes">
 *        <div class="TabPage">
 *          <div class="Dialog ui-helper-clearfix">
 *            <div class="col-xs-4 WideBottomMargin BottomPaddingWide">
 *              <label for="ClassID" class="RightPadding">{ProductType}</label>
 *              <select name="ClassID" id="ClassID" >
 *                <option value="default">{SelectProductTypeClass}</option>
 *              </select>
 *            </div>
 *
 *            <div id="NewProductTypeNameInput" class="col-xs-5">
 *              <label for="Alias" class="RightPadding">{Name} <span class="MandatoryStar">*</span></label>
 *              <input type="text" class="Mandatory ep-js" data-js=".uiInput()" size="20" id="Alias" name="Alias" value="" class="AlignMiddle" placeholder="{Example}: {TShirt}" />
 *            </div>
 *
 *            <div id="ep-step2-wrapper" class="#IF(NOT ((#Class.Alias NE "ProductClass") OR (#COUNT(#SelectedVariations)))) HideElement#ENDIF">
 *                <div id="AttributeContainer">
 *                    <div class="row ui-helper-clearfix ep-variationassistant-attributecontainer" data-attributecount="0" >
 *                      <div class="col-xs-12">
 *                        <div class="row">
 *                          <div class="col-xs-4 TopMargin">
 *                            <span class="ep-variationassistant-isusedcheckbox">
 *                              <input type="checkbox" checked="checked" disabled="disabled" class="ep-variationassistant-usedcheckbox" >
 *                              <input type="hidden" name="UseAttribute0" class="ep-variationassistant-useattribute" value="1">
 *                            </span>
 *                            <h5>{UnityAttributeName}</h5>
 *                          </div>
 *                          <div class="col-xs-12"></div>
 *                        </div>
 *                        <div class="row">
 *                          <div class="col-xs-4">
 *                            <input type="text" name="VariationAttributeAlias0" class="ep-variationassistant-attributealias ep-js" data-js=".uiValidate({setManually: 'valid', showErrorMsgAlways: true})" data-valid="true" placeholder="{Example}:&nbsp;{Color}" >
 *                            <input type="hidden" name="TypePackage0" value="PreDefLocalizedString">
 *                          </div>
 *                        </div>
 *                        <div class="ep-variationassistant-details">
 *                          <div class="row">
 *                            <div class="col-xs-12 WideTopMargin">
 *                              <h5>{UnityAttributeValues}</h5>
 *                            </div>
 *                          </div>
 *                          <div class="ep-variationassistant-attributelist">
 *                            <div class="row ui-helper-clearfix">
 *                              <div class="col-xs-4 NoWrap TopTinyMargin">
 *                                <input class="jsInput TextSlim FullWidth ep-variationassistant-attributevalueinput-initial ep-js" data-js=".uiValidate({setManually: 'valid', showErrorMsgAlways: true})" data-valid="true" name="VariationAttributeValues0" type="text" placeholder="{Example}:&nbsp;{Blue}" >
 *                                <i class="CursorPointer ep-placeInsideInput HideElement fa fa-trash fa-lg" title="{DeleteAttributeValue}"></i>
 *                              </div>#REM <!-- .col-xs-12 --> #ENDREM
 *                            </div>#REM <!-- .row --> #ENDREM
 *                          </div>
 *
 *                          <div class="row ui-helper-clearfix">
 *                            <div class="col-xs-4 NoWrap TopTinyMargin">
 *                              <input class="jsInput TextSlim FullWidth ep-variationassistant-attributevalueinput" type="text" placeholder="{Example}:&nbsp;{Red}">
 *                            </div>#REM <!-- .col-xs-12 --> #ENDREM
 *                          </div>#REM <!-- .row --> #ENDREM
 *                        </div>
 *                      </div>#REM <!-- .col-xs-12 --> #ENDREM
 *                    </div>#REM <!-- .row --> #ENDREM
 *                </div>
 *
 *                <div class="row">
 *                  <div class="col-xs-12 WideTopMargin">
 *                    <label class="ep-button ep-button-add-option WideTopMargin" id="EnterNewAttributeButton">
 *                      <div title="{NewAttribute}">
 *                        <i class="fa fa-plus MarginRight position-relative">
 *                          <span class="ep-blend-layer"></span>
 *                        </i>
 *                        <em class="batch-action-title">{NewAttribute}</em>
 *                      </div>
 *                    </label>
 *                  </div>#REM <!-- .col-xs-12 --> #ENDREM
 *                </div>#REM <!-- .row --> #ENDREM
 *              </div>
 *            #ENDLOCAL
 *          </div>
 *        </div>
 *      </div>
 *    <!-- EnterAttributesArea: end -->
 *
 *    <!-- Select Default Area: start-->
 *      <div id="DefaultList" class="HideElement">
 *        <div class="TabPage" >
 *          <div class="Dialog">
 *            <div class="ep-notification-success BottomWideMargin">
 *              <h3>{AddedNewAttribute}</h3>
 *              <p>{PleaseSelectDefaultAttributeValues}</p>
 *            </div>#REM <!-- .ep-notification-success --> #ENDREM
 *            <div id="SelectDefaultContainer" class="TopWideMargin">
 *
 *            </div>
 *          </div>
 *        </div>
 *      </div>
 *    <!-- Select Default Area: end-->
 *
 *    <!-- ProductTableArea: start -->
 *    <div id="ProductTable" class="HideElement">
 *      <div class="TabPage" >
 *        <div class="Dialog">
 *          <div id="SelectProductsInfo" class="ep-assistant-box BottomWideMargin">
 *            <h3>{SelectProducts}</h3>
 *            #IF(#COUNT(#SubProducts))<p>{SelectProductsInfo}</p>#ENDIF
 *          </div>#REM <!-- .ep-assistant-box --> #ENDREM
 *
 *          <div id="ProductListContainer">
 *
 *          </div>
 *        </div>
 *      </div>
 *    </div>
 *    <!-- ProductTableArea: end -->
 *
 *
 *
 * @class jQuery.ui.productUiVariationwizardUnity
 *
 * @uses de_epages
 * @uses de_epages.product.ui.variationattribute-unity
 * @uses ep
 * @uses ep.dict
 * @uses ep.ui.validate
 * @uses jQuery.metaparse
 * @uses jQuery.tmpl
 * @uses jquery.ui.widget
 * @since 7.15.0
 */

/**
 * @cfg {String} productTypeSelector            Selector for dropdown for choosing a product type
 * @cfg {String} productTypeAliasInput          Selector for product type alias input
 * @cfg {String} productTypeAliasInputWrapper   Selector for element containing product type alias
 * @cfg {String} attributeArea                  Selector for attribute selection area
 * @cfg {String} attributeAreaHeadline          Selector for attribute selection area headline
 * @cfg {String} attributeStep                  Selector for element containing everything attribute/product type related (Step1)
 * @cfg {String} attributes                     Element containing attribute containers for user defined attributes
 * @cfg {String} attributeContainer             Selector for attribute containers
 * @cfg {String} attributeAlias                 Selector for attribute alias inputs
 * @cfg {String} attributeValues                Selector for attribute value list
 * @cfg {String} deletedAttributesArea          Selector for the deleted attributes area
 * @cfg {String} deleteAttribute                Selector for delete attribute icon
 * @cfg {String} noNewAttributesError           Selector for error box informing the merchant about the feature limit
 * @cfg {String} isUsed                         Selector for select attribute checkbox
 * @cfg {String} newAttributeButton             Selector for new attribute button
 * @cfg {String} defaultList                    Selector for element containing everything for the default list (Step 1.5)
 * @cfg {String} selectDefaultContainer         Selector for element containing the default attribute choices
 * @cfg {String} selectAllValues                Selector for the select all toggle checkbox
 * @cfg {String} selectValue                    Selector for the select attribute value checkbox
 * @cfg {String} productTable                   Selector for element containing everything for the selection of variation product (Step 2)
 * @cfg {String} productListContainer           Selector for the mount point for the product rows
 * @cfg {String} backButton                     Selector for the back button
 * @cfg {String} nextButton                     Selector for the next button
 * @cfg {String} finishButton                   Selector for the finish button
 * @cfg {String} bus                            Selector for the bus element to communicate with the variationattribute-widgets
 *
 * @cfg {String} baseAlias                      base alias for creating the new variations
 * @cfg {String} baseName                       base name for creating the new variations
 *
 * @cfg {String} inputsModifyingCountOfGeneratedVariationsToShowVariationCountErrorOn
 *                                              Selector for error box informing the merchant about the feature limit
 *
 * @cfg {Array} existingProducts                array of all existing products as objects
 *                                                Format: {
 *                                                    alias: Alias1,
 *                                                    name: Name1,
 *                                                    attributes: {
 *                                                        attribute1: attributeValue2
 *                                                    }
 *                                                }
 *
 * @cfg {Number} attributeCount                 count of predefined attributes
 * @cfg {Number} maxVariations                  maximal number of variations allowed
 * @cfg {Number} maxAttributes                  maximal number of own attributes
 * @cfg {Number} maxAttributeValues             maximal number of attribute values
 * @cfg {Number} currentClass                   Id of currently loaded class
 * @cfg {Number} basicUrl                       URL for page reload with new classID

 */

/*
 * @copyright       © Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages/product/ui/variationwizard-unity
 */


/*jslint nomen: true*/
/*global define*/

define('de_epages/product/ui/variationwizard-unity', [
    'jquery',
    'ep',
    'de_epages',
    'ep/ui/simple-tooltip',
    '$tmpl!./variationwizard-unity-newattribute',
    '$tmpl!./variationwizard-unity-selectdefault',
    '$tmpl!./variationwizard-unity-selectvariationstable',
    '$dict!../dictionary',

    'de_epages/product/ui/variationattribute-unity',
    'jquery/ui/widget',
    'jquery/metaparse',
    'ep/fn/holdontop',
    'ep/ui/validate',
    'ep/ui/richtable'
], function ($, ep, de_epages, simpleTooltip, tmplAttribute, tmplSelectDefault, tmplSelectVariations, dict) {
    /*
     * @dictionary      ep.dict
     *
     * @translations    {Color}
     *                  {ItemNoShort}
     *                  {Name}
     *                  {SelectAll}
     *
     * @dictionary      de_epages.product.dict
     *
     * @translations   {Example}
     *                 {Blue}
     *                 {Red}
     *                 {UnityMoreAttributesThanAllowed}
     *                 {UnityAttributeName}
     *                 {UnityAttributeValues}
     *                 {UnityUseThisAttribute}
     *                 {ProductAttributeExists}
     *                 {UnityDuplicateProductAlias}
     *                 {NoProductAlias}
     *                 {UnityAttributeXMLValidationMessage}
     *                 {MoreNewProductsThanAllowed}
     *                 {EmptyAttributeAlias}
     *                 {DeleteAttribute}
     *                 {DeleteAttributeValue}
     *                 {UnityTooLittleAttributeValues}
     */

    // translation object
    var translations = {
        ean: dict.translate('ItemNoShort'),
        name: dict.translate('Name'),
        color: dict.translate('Color'),
        selectAll: dict.translate('SelectAll'),

        moreAttributesThanAllowed: dict.translate('UnityMoreAttributesThanAllowed'),
        example: dict.translate('Example'),
        blue: dict.translate('Blue'),
        red: dict.translate('Red'),
        attributeName: dict.translate('UnityAttributeName'),
        attributeValue: dict.translate('UnityAttributeValues'),
        useThisAttribute: dict.translate('UnityUseThisAttribute'),
        doubleAttributeAlias: dict.translate('ProductAttributeExists'),
        doubleVariationAlias: dict.translate('UnityDuplicateProductAlias'),
        noAlias: dict.translate('NoProductAlias'),
        emptyAlias: dict.translate('EmptyAttributeAlias'),
        xmlComplaint: dict.translate('UnityAttributeXMLValidationMessage'),
        tooManyVariations: dict.translate('MoreNewProductsAsPossible'),
        tooManyVariationProducts: dict.translate('MoreNewProductsThanAllowed'),
        deleteAttribute: dict.translate('DeleteAttribute'),
        deleteAttributeValue: dict.translate('DeleteAttributeValue'),
        tooLittleAttributeValues: dict.translate('UnityTooLittleAttributeValues')
    };

    $.widget('ui.productUiVariationwizardUnity', {
        timeout: {},

        options: {
            // productTypeSelector
            // productTypeAliasInput
            // productTypeAliasInputWrapper
            // attributeArea
            // attributeAreaHeadline
            // deletedAttributesArea
            // attributes
            // attributeAlias
            // attributeStep
            // attributeContainer
            // attributeCount
            // attributeValues
            // deleteAttribute
            // isUsed
            // newAttributeButton
            // defaultList
            // selectDefaultContainer
            // productTable
            // productListContainer
            // backButton
            // nextButton
            // finishButton
            // existingProducts
            // bus
            // baseAlias
            // baseName
            // inputsModifyingCountOfGeneratedVariationsToShowVariationCountErrorOn
            maxVariations: 1000,
            maxVariationsTooltip: '',
            maxAttributes: 3,
            maxAttributeValues: 10
        },

        currentPage: 0,

        valid: true,

        lastUsedAttributeOrValueCheckbox: $('<div />'),

        _create: function() {
            var self = this,
                elem = self.element,
                o = self.options;

            self.lastUsedAttributeOrValueCheckbox = $(o.selectValue).first();

            // prepare translations
            translations.maxVariationsTooltip = o.maxVariationsTooltip;

            // buttons
                self.nextButton = $(o.nextButton);
                self.backButton = $(o.backButton);
                self.finishButton = $(o.finishButton);

            // product type selection
                self.productTypeAliasInput = $(o.productTypeAliasInput);
                self.productTypeAliasInputWrapper = $(o.productTypeAliasInputWrapper);
                self.productTypeSelector = $(o.productTypeSelector);

            // attribute list
                self.attributeArea = $(o.attributeArea);
                self.attributeAreaHeadline = $(o.attributeAreaHeadline);
                self.attributeList = $(o.attributes);
                self.newAttributeButton = $(o.newAttributeButton);
                self.attributeStep = $(o.attributeStep);
                self.deletedAttributesArea = $(o.deletedAttributesArea);
                self.potentialVariationCountOutput = $(o.potentialVariationCountOutput);

            // default step
                self.defaultList = $(o.defaultList);
                self.defaultContainer = $(o.selectDefaultContainer);

            // select variations step
                self.productTable = $(o.productTable);
                self.productListContainer = $(o.productListContainer);


            // product type
                // handle and validate input of alias for a new product type
                self.productTypeAliasInput.on('input', self._delay('alias', self._handleProductTypeAliasInput).bind(self));
                self.productTypeSelector.on('change', self._handleProductTypeSwitch.bind(self));

            // attributes
                self.attributeList.on('click', o.deleteAttribute, self._deleteAttribute.bind(self));
                self.attributeArea.on('change', o.isUsed, self._toggleAttributeUsage.bind(self));
                self.newAttributeButton.on('click', self._newAttribute.bind(self));
                self.attributeList.on('mousedown', o.inputsModifyingCountOfGeneratedVariationsToShowVariationCountErrorOn, function(evt) {
                    self.lastUsedAttributeOrValueCheckbox = $(this);
                });
                self.attributeArea.on('input', '.ep-variationassistant-attributealias', self._delay('alias', function(evt) {
                    var self = this;

                    self._validateAttributeAliases.bind(self)(true);
                    self._validateSingleAttribute.bind(self)($(evt.target).closest(o.attributeContainer), true);
                }).bind(self));

            // default
                self.defaultList.on('change', 'input', self._validateDefaultSelection.bind(self));

            // select products
                self.productListContainer.on('input', 'input.ep-variationassistant-newvariationalias', self._validateNewVariations.bind(self));
                self.productListContainer.on('change', 'input.ep-variationassistant-selectnewvariation', self._validateNewVariations.bind(self));
                self.productListContainer.on('change.uiRichtable', 'input.ep-variationassistant-selectnewvariation, th.Checkbox > input[name="selector"]', self._delay('richtable', self._validateNewVariations).bind(self));

            // general
                self.nextButton.on('click', self._handleNext.bind(self));
                self.backButton.on('click', self._handleBack.bind(self));
                self.finishButton.on('click', self._handleSubmit.bind(self));
                $(o.bus).on('triggerValidation.AttributeWizard', self._delay('validate', self._validateAttributes, 10).bind(self));

                // prevent accidental submission of form by pressing enter in an input
                elem.on('keydown', function(evt) {
                    if (evt.which == 13 || evt.keyCode == 13) {
                        evt.preventDefault();
                        evt.stopPropagation();
                        return false;
                    }
                });

            // initial validation
            self._validateAttributes();
        },

        _handleProductTypeSwitch: function (evt) {
            var self = this,
                o = self.options,
                val = evt.target.value,
                options = self.productTypeSelector.prop('options'),
                // get index of currently loaded class
                currentClassIndex = options.selectedIndex,
                isNew = $(options[currentClassIndex]).data('isnew'),
                newProductTypeAliasIsEmpty = self.productTypeAliasInput.val() !== undefined && self.productTypeAliasInput.val().length === 0,
                // get number of dirty inputs with content
                changed = self.element.find('input[type="text"].ui-changed').filter(function(index, input) {
                    return input.value && input.value.length;
                }).length;

            if (!changed) {
                self.element.uiForm({showSaveWarn:false});
            }

            switch(val) {
                case 'default':
                    // reset ui
                    self.productTypeAliasInputWrapper.addClass('HideElement');
                    self.attributeArea.addClass('HideElement');
                    break;
                case self.options.currentClass + '':
                    // restore ui

                    if (!isNew || isNew && !newProductTypeAliasIsEmpty) {
                        self.attributeArea.removeClass('HideElement');
                    }

                    if (isNew) {
                        self.productTypeAliasInputWrapper.removeClass('HideElement');
                    }
                    break;
                default:
                    if (isNew && newProductTypeAliasIsEmpty) {
                        // dont show save warning even if alias input was touched
                        self.element.uiForm({showSaveWarn:false});
                    }

                    // trigger reload with new ClassID
                    window.location = o.basicUrl + '&Class=' + val;
                    break;
            }
        },

        _handleProductTypeAliasInput: function (evt) {
            var self = this,
                inputVal = evt.target.value;

            self.attributeArea[inputVal.length ? 'removeClass' : 'addClass']('HideElement');
            self.nextButton[inputVal.length ? 'removeClass' : 'addClass']('HideElement');
            self.attributeAreaHeadline.html('').text(inputVal);
        },

        // attribute list
        _deleteAttribute: function (evt) {
            var self = this,
                o = self.options,
                container = $(evt.target).closest(o.attributeContainer),
                isUsed = container.find('.ep-variationassistant-useattribute');

            isUsed.val(0);
            self.deletedAttributesArea.append(isUsed);

            container.productUiVariationattributeUnity('destroy');
            container.remove();
            self._validateAttributes();

            if ($(o.isUsed).filter(':checked').length < o.maxAttributes && self.attributeList.children().length < o.maxVariationAttributes) {
                // show button
                self.newAttributeButton.closest('.row').removeClass('HideElement');
            }
        },

        _toggleAttributeUsage: function(evt) {
            var self = this,
                o = self.options,
                target = $(evt.target),
                attributeValues = target.closest(o.attributeContainer).find(o.attributeValues),
                output = target.parent().find('.ep-variationassistant-useattribute'),
                active = evt.target.checked;

            // activate/deactivate inputs
            attributeValues.find('input').prop('disabled', !active);

            if (active) {
                // hide / show attributeValues area
                attributeValues.removeClass('HideElement');
                output.val(1);
            } else {
                simpleTooltip.hide();
                // hide / show attributeValues area
                attributeValues.addClass('HideElement');
                output.val(0);
            }

            self._validateAttributes();
        },

        _newAttribute: function () {
            var self = this,
                o = self.options,
                template;

            if ($(o.isUsed).filter(':checked').length < o.maxAttributes && self.attributeList.children().length < o.maxVariationAttributes) {
                template = tmplAttribute({
                        count: o.attributeCount,

                        tExample: translations.example,
                        tDeleteAttribute: translations.deleteAttribute,
                        tDeleteAttributeValue: translations.deleteAttributeValue,
                        tBlue: translations.blue,
                        tRed: translations.red,
                        tColor: translations.color,
                        tUnityAttributeName: translations.attributeName,
                        tUnityAttributeValues: translations.attributeValue,
                        tUnitySelectAll: translations.selectAll,
                        tUseThisAttribute: translations.useThisAttribute
                    });

                o.attributeCount++;

                template.productUiVariationattributeUnity({
                    input: '.ep-variationassistant-attributevalueinput',
                    initialInput: '.ep-variationassistant-attributevalueinput-initial',
                    list: '.ep-variationassistant-attributelist',
                    bus: o.bus,
                    maxValues: o.maxAttributeValues,
                    selectAllValues: o.selectAllValues,
                    selectValue: o.selectValue
                });

                template.find('.ep-js').metaparse();

                self.attributeList.append(template);

                if (self.attributeList.children().length >= o.maxVariationAttributes || $(o.isUsed).filter(':checked').length >= o.maxAttributes) {
                    // hide button if this was the last allowed attribute
                    self.newAttributeButton.closest('.row').addClass('HideElement');
                    $(o.noNewAttributesError).addClass('HideElement');
                }

            } else {
                // hide button if there are 3 or more Attributes already
                self.newAttributeButton.closest('.row').addClass('HideElement');
                $(o.noNewAttributesError).addClass('HideElement');
            }
        },

        _validateAttributeCount: function (ignoreValidation) {
            var self = this,
                o = self.options,
                attributeCheckboxes = $(o.isUsed),
                checkedAttributes = attributeCheckboxes.filter(':checked'),
                nonEmptyCheckedAttributes = checkedAttributes.closest(o.attributeContainer).filter(function(index, element) {
                    // get liststate and check length of alias
                    return !($(element).productUiVariationattributeUnity('isEmpty') && !$(element).find(o.attributeAlias).val().length);
                }),
                valid = true;

            // check if maximum number of attributes is reached and disable/enable checkboxes
            if (checkedAttributes.length >= o.maxAttributes) {
                // disable checkboxes
                attributeCheckboxes.each(function(index, element) {
                    if (!element.checked) {
                        element.disabled = true;
                    }
                });

                // add hover events to display a warning
                attributeCheckboxes.filter(':not(:checked)').closest('span').on('mouseover.variationAssistantWarning', function() {
                    var $elem = ep(this);

                    simpleTooltip.show({
                        element: $elem,
                        message: translations.moreAttributesThanAllowed,
                        event: 'click',
                        eventtype: 'none'
                    });
                }).on('mouseleave.variationAssistantWarning', simpleTooltip.hide);

                // hide add new attribute button if max number is chosen (new attribute would be checked automatically and cause an error)
                self.newAttributeButton.closest('.row').addClass('HideElement');

                // if there are more than 3 attributes checked, set invalid
                if (checkedAttributes.length > o.maxAttributes) {
                    valid = false;
                }
            } else {
                if ($(o.isUsed).filter(':checked').length < o.maxAttributes && self.attributeList.children().length < o.maxVariationAttributes) {
                    // show add new attribute button if less than 3 attributes are checked and less than feature limit attributes are created
                    self.newAttributeButton.closest('.row').removeClass('HideElement');
                }
                // do not toggle checkboxes that are from new attributes or that are used
                attributeCheckboxes.filter(':not(.ep-variationassistant-newattributecheckbox, .ep-variationassistant-usedcheckbox)').prop('disabled', false);


                // remove hover events
                attributeCheckboxes.closest('span').off('.variationAssistantWarning');
            }

            // prevent use of no attribute
            if (nonEmptyCheckedAttributes.length <= 0) {
                valid = false;
            }

            if (valid !== self.valid && !ignoreValidation) {
                self._validateAttributes();
            }

            return valid;
        },

        _validateAttributeAliases: function (validate) {
            var self = this,
                o = self.options,
                elem = self.element,

                valid = true,
                valueElements = elem.find(o.attributeAlias),
                testAlias = /^[^<>&%"'0-9][^<>&%"'][^$|\s+]*$/,
                message,
                doubleAlias;

            valueElements.each(function(index, element) {
                var $element = $(element),
                    hasValidate = $element.data('uiUiValidate'),
                    elementValue = element.value;

                if (elementValue.length && !elementValue.match(testAlias)) {
                    valid = false;
                    if (hasValidate) {
                        $element.uiValidate('option', {'customMessage': translations.xmlComplaint});
                        $element.data('valid', false)
                            .trigger('validate.uiValidate')
                            .addClass('DialogError');
                    }

                } else if (hasValidate) {
                    $element.removeClass('DialogError')
                        .data('valid', true)
                        .trigger('validate.uiValidate');
                }
            });

            doubleAlias  = self._findDuplicateVariationAlias(valueElements);

            if (doubleAlias.length) {
                doubleAlias.each(function(index, element) {
                    var $element = $(element),
                        hasValidate = $element.data('uiUiValidate');

                    valid = false;
                    if (hasValidate) {
                        $element.uiValidate('option', {'customMessage': translations.doubleAttributeAlias});
                        $element.data('valid', false)
                            .trigger('validate.uiValidate')
                            .addClass('DialogError');
                    }
                });
            }

            // validate everything if validity is different from general validity or if alias was emptied
            if (valid !== self.valid && validate === true) {
                self._validateAttributes();
            }

            return valid;
        },

        _validateSingleAttribute: function(element, validate) {
            var self = this,
                o = self.options,
                $element = $(element),
                attributeAlias = $element.find(o.attributeAlias),
                valid = false;

            // is value list empty?
            if ($element.productUiVariationattributeUnity('isEmpty')) {

                // does the alias have a length?
                if (attributeAlias.val().length) {
                    // throw error
                    valid = false;
                } else {
                    // if empty alias and empty list, remove error
                    $element.productUiVariationattributeUnity('removeEmptyWarning');
                    attributeAlias.uiValidate('option', {'valid': true});
                    attributeAlias.removeClass('DialogError');
                    valid = true;
                }
            } else {
                $element.productUiVariationattributeUnity('removeEmptyWarning');

                if (!attributeAlias.val().length) {
                    // set error message
                    attributeAlias.uiValidate('option', {'customMessage': translations.emptyAlias, 'valid': 0});
                    attributeAlias.addClass('DialogError');

                    valid = false;
                } else {
                    valid = $element.productUiVariationattributeUnity('isValid');
                }
            }

            // validate everything if validity is different from general validity or if alias was emptied
            if (valid !== self.valid && validate === true) {
                self._validateAttributes();
            }

            return valid;
        },

        _validateAttributes: function(evt) {
            var self = this,
                o = self.options,
                elem = self.element,
                usedAttributes = $.makeArray(elem.find(o.isUsed).filter(':checked').closest(o.attributeContainer)),
                valid = true,
                attributeCheck,
                selectedAttributes,
                potentialVariationCount;


            if (self._validateAttributeCount(true) && self._validateAttributeAliases()) {
                // if 0 < used attributes <= max value attributes are selected, check each selected attribute for errors
                valid = usedAttributes.every(self._validateSingleAttribute.bind(self));

            } else {
                valid = false;
            }

            selectedAttributes = self._getSelectedAttributes();

            potentialVariationCount = selectedAttributes.reduce(function(accumulator, attribute) {
                if (!attribute.values.length) {
                    return accumulator;
                }

                return accumulator ? attribute.values.length * accumulator : attribute.values.length;
            }, 0);

            if (!potentialVariationCount) {
                // display error message
                simpleTooltip.show({
                    element: self.lastUsedAttributeOrValueCheckbox,
                    message: translations.tooLittleAttributeValues,
                    orientation: 'middle left',
                    event: 'click',
                    eventtype: 'none',
                    additionalClass: 'ep-formfield-tooltip ep-uiValidate-info ep-variationassistant-errortooltip-left'
                });
                valid = false;
            } else if (potentialVariationCount > o.maxVariations) {
                // display error message
                simpleTooltip.show({
                    element: self.lastUsedAttributeOrValueCheckbox,
                    message: translations.maxVariationsTooltip,
                    orientation: 'middle left',
                    event: 'click',
                    eventtype: 'none',
                    additionalClass: 'ep-formfield-tooltip ep-uiValidate-info ep-variationassistant-errortooltip-left'
                });
                valid = false;
            } else {
                simpleTooltip.hide();
            }
            self.potentialVariationCountOutput.html(potentialVariationCount);

            self.nextButton.prop('disabled', !valid);

            self.valid = valid;
            return valid;
        },

        // assign new attribute to existing products
        // quick check
        _checkForUnassignedAttributes: function() {
            var self = this,
                o = self.options,
                attributes = self._getSelectedAttributes().map(function(attribute) {
                    return attribute.alias;
                });

            // for all existing products get all used attributes and check if any selected attribute is not included
            // returns true if one attribute is unassigned to at least one product
            return o.existingProducts.some(function(product) {
                var usedAttributes = Object.keys(product.attributes).sort();

                return attributes.some(function(attribute) {
                    return (usedAttributes.indexOf(attribute) === -1);
                });
            });
        },

        // get array of unassigned attributes (slower than check above)
        _getUnassignedAttributes: function() {
            var self = this,
                o = self.options,
                attributes = self._getSelectedAttributes(),
                attributeNames = attributes.map(function(attribute) {
                        return attribute.alias;
                    }),
                unassignedAttributesNames;

            unassignedAttributesNames = o.existingProducts.reduce(function(accumulator, product) {
                var usedAttributes = Object.keys(product.attributes).sort(),
                    filteredAttributes = attributes;

                attributeNames.forEach(function(attribute) {
                    if (usedAttributes.indexOf(attribute) === -1) {
                        // push unused attributes into accumulator
                        accumulator.push(attribute);

                        // avoid iterating over already recognized unused attributes by removing them from the array
                        filteredAttributes = filteredAttributes.filter(function(attribute) {
                            return (accumulator.indexOf(attribute) === -1);
                        });
                    }
                });

                // write filtered list to global variable (doing so inside the loop may cause unpredictable results)
                attributes = filteredAttributes;
                return accumulator;
            }, []);

            return attributes.filter(function(attribute) {
                return (unassignedAttributesNames.indexOf(attribute.alias) !== -1);
            });
        },

        _getSelectedAttributes: function() {
            var self = this,
                o = self.options,
                elem = self.element,
                selectedAttributes = $.makeArray(elem.find(o.isUsed).filter(':checked').closest(o.attributeContainer));

            return selectedAttributes.reduce(function(accumulator, element) {
                    var $element = $(element),
                        content = $element.find(o.attributeAlias).val();

                if (!$element.productUiVariationattributeUnity('isEmpty')) {
                    accumulator.push({
                        alias: content,
                        values: $element.productUiVariationattributeUnity('getSelectedValues', '.ep-variation-attribute-alias'),
                        texts: $element.productUiVariationattributeUnity('getSelectedValues', '.ep-variation-attribute-value'),
                        count: $element.data('attributecount')
                    });
                }

                return accumulator;
            }, []);
        },

        _generateAttributeAssignmentTable: function() {
            var self = this,
                unassignedAttributes = self._getUnassignedAttributes();

            self.defaultContainer.empty();
            unassignedAttributes.forEach(self._generateSingleAttributeAssingement.bind(self));
        },

        _generateSingleAttributeAssingement: function(attribute) {
            var self = this,
                template = tmplSelectDefault({
                    tHeadline: translations.attributeName,
                    attributeAlias: attribute.alias,
                    attributeValues: attribute.values,
                    count: attribute.count
                });

            self.defaultContainer.append(template);
        },

        _validateDefaultSelection: function() {
            var self = this,
                valid = $.makeArray(self.defaultContainer.children()).every(function(element) {
                    return $(element).find('input:checked').length;
                });

            self.nextButton.prop('disabled', !valid);

            return valid;
        },

        _getDefaultValues: function() {
            var self = this;

            return $.makeArray(self.defaultContainer.children()).reduce(function(accumulator, element) {
                var selectedInput = $(element).find('input:checked');

                accumulator[selectedInput.closest('.ep-variationassistant-attributevaluestable').data('attributename')] = selectedInput.val();


                return accumulator;
            }, {});
        },

        // Variation products list
        _generateVariationList: function() {
            var self = this,
                o = self.options,
                selectedAttributes = self._getSelectedAttributes(),
                defaults = self._getDefaultValues(),
                existingProducts = o.existingProducts.map(function(product) {
                    Object.keys(defaults).forEach(function(_key) {
                        // add attribute's value
                        product.attributes[_key] = defaults[_key];
                        // add attribute's NameOrAlias
                        product.attributes[_key + '_text'] = defaults[_key];
                    });

                    return product;
                }),
                newProducts = self._generateNewProducts(selectedAttributes, existingProducts);

            var template = tmplSelectVariations({
                    selectedAttributes: selectedAttributes,
                    existingProducts: existingProducts,
                    defaults: self._getDefaultValues(),
                    productName: o.baseName,
                    newProducts: newProducts,
                    maxVariations: o.maxVariations,
                    tArticleNumber: translations.ean,
                    tName: translations.name
                });

            self.variationListElements = $.extend({}, template.tmplItem('elements'));

            self.variationListElements.warningMessage.html(translations.tooManyVariationProducts);
            template.find('#ep-variationassistant-maxvariations').html(o.maxVariations);

            // assync to prevent blockage for high number of variations
            setTimeout(function() {
                self.variationListElements.metaparse.metaparse();
                self.variationListElements.aliasInput.metaparse();
                self.variationListElements.variationCountWarning.holdOnTop('top');

                self._validateNewVariations();
            }, 1);

            self.productListContainer.html('').append(template);
            // reset position to top
            window.scrollTo(0, 0);
        },

        _generateNewProducts: function(selectedAttributes, existingProducts) {
            var self = this,
                o = self.options,
                defaults = self._getDefaultValues(),
                selectedAttributeAliases = selectedAttributes.map(function(attribute) {
                    return attribute.alias;
                }),
                existingAliases = existingProducts.map(function(product) {
                    return product.alias;
                }),
                baseAlias = o.baseAlias,

                count = 1,
                newProducts,
                possibleProducts,
                possibleCombinationStrings,
                existingCombinationStrings,

                // generate next alias by combining next alias with 4 digit number
                generateNextAlias = function() {
                    var alias = baseAlias + '-' + ('0000' + count).slice(-4);

                    // if alias is already in use, create new alias
                    while (existingAliases.indexOf(alias) !== -1) {
                        count++;
                        alias = baseAlias + '-' + ('0000' + count).slice(-4);
                    }

                    count++;
                    // add new alias to list and return new alias
                    existingAliases.push(alias);
                    return alias;
                };

            // loop over each attribute to create an array containing all possible products
            possibleProducts = selectedAttributes.reduce(function(accumulator, attribute) {
                var temp = [],
                    obj;

                // create new layer using old layer as template for each value
                attribute.values.forEach(function(value, index) {
                    if (accumulator.length) {
                        // for each value copy products from last iteration and append new attribute + value
                        temp = temp.concat(accumulator.map(function(product) {
                            newProducts = $.extend({}, product);
                            newProducts[attribute.alias] = value;
                            newProducts[attribute.alias + '_text'] = attribute.texts[index];

                            return newProducts;
                        }));

                    // if first iteration just create new Objects
                    } else {
                        obj = {};
                        obj[attribute.alias] = value;
                        obj[attribute.alias + '_text'] = attribute.texts[index];

                        temp.push(obj);
                    }
                });

                return temp;
            }, []);

            // create list of all valuecombinations possible with the selected attributes
            possibleCombinationStrings = possibleProducts.map(function(product) {

                return selectedAttributeAliases.reduce(function(accumulator, alias) {
                        accumulator += product[alias];
                        return accumulator;
                    }, '');
            });

            // map existing products to  valuecombinations as well
            existingCombinationStrings = existingProducts.map(function(product) {

                return selectedAttributeAliases.reduce(function(accumulator, alias) {
                        accumulator += product.attributes[alias];
                        return accumulator;
                    }, '');
            });

            // to find existing products compare valuecombinations of each product against existing combinations
            newProducts = possibleProducts.filter(function(product, index) {

                return (existingCombinationStrings.indexOf(possibleCombinationStrings[index]) === -1);
            });

            // add aliases to filtered list (added only now to avoid gaps)
            newProducts.map(function(product) {
                product.alias = generateNextAlias();

                return product;
            });

            return newProducts;
        },

        _validateNewVariations: function() {
            var self = this,
                o = self.options,
                valid = true,
                selectedVariations = self._getSelectedVariations(),
                duplicates = self._findDuplicateVariationAlias(selectedVariations),
                emptyAliases = self._findEmptyVariationAlias(selectedVariations);

            // remove errors
            self.variationListElements.aliasInput
                .filter('.ui-invalid, DialogError')
                .data('valid', true)
                .trigger('validate.uiValidate')
                .removeClass('DialogError');

            // check for duplicate aliases
            if (duplicates.length) {
                duplicates.addClass('DialogError');
                duplicates.each(function(index, element) {
                    var $element = $(element);
                    if ($element.is('span')) {
                        return;
                    }
                    $element.uiValidate('option', {'customMessage': translations.doubleVariationAlias});
                });
                duplicates.data('valid', false).trigger('validate.uiValidate');
                valid = false;
            }

            // check for empty aliases
            if (emptyAliases.length) {
                emptyAliases.addClass('DialogError');
                emptyAliases.each(function(index, element) {
                    var $element = $(element);
                    if ($element.is('span')) {
                        return;
                    }
                    $element.uiValidate('option', {'customMessage': translations.noAlias});
                });
                emptyAliases.data('valid', false).trigger('validate.uiValidate');
                valid = false;
            }

            self.variationListElements.variationCountWarning.addClass('HideElement');

            // check whether to many products were selected (generally should not be possible anymore)
            if (o.existingProducts.length + selectedVariations.length > o.maxVariations) {
                self.variationListElements.variationCountWarning.removeClass('HideElement');
                valid = false;
            }

            // write count of selected variation products to dom
            self.potentialVariationCountOutput.html(o.existingProducts.length + selectedVariations.length);

            self.finishButton.prop('disabled', !valid);
            return valid;
        },

        _findDuplicateVariationAlias: function(selectedVariations) {
            var self = this,
                aliasInputs = selectedVariations ? selectedVariations : self._getSelectedVariations(),
                duplicates;
            // check if the attribute value is already used in this attribute
            // sort array so only neccessary to compare with next/previous element in array
            aliasInputs.sort(function(a, b) {
                var $a = $(a),
                    $b = $(b),
                    contentA = $a[$a.is('input') ? 'val' : 'text'](),
                    contentB = $b[$b.is('input') ? 'val' : 'text']();

                return contentA.localeCompare(contentB);
            });

            // filter all elements and only leave those that are same as next or previous element
            duplicates = aliasInputs.filter(function(index, element) {
                var $a = $(element),
                    $b = $(aliasInputs[index - 1]),
                    $c = $(aliasInputs[index + 1]),
                    contentA = $a[$a.is('input') ? 'val' : 'text'](),
                    contentC = $c[$c.is('input') ? 'val' : 'text'](),
                    contentB = $b[$b.is('input') ? 'val' : 'text']();

                return (contentA.length && (contentA.toLowerCase() === contentB.toLowerCase() || contentA.toLowerCase() === contentC.toLowerCase()));
            });

            return $(duplicates);
        },

        _findEmptyVariationAlias: function(selectedVariations) {
            var self = this,
                empty;

            if (selectedVariations) {
                empty = $(selectedVariations.filter(function(index, element) {
                    return !element.value.length;
                }));

            } else {
                empty = $(self.variationListElements.aliasInput.filter(function(index, element) {
                    // if variation is selected
                    if ($(element).closest('tr').find('input.ep-variationassistant-selectnewvariation').prop('checked')){
                        return !element.value.length;
                    }

                    return false;
                }));
            }

            return empty;
        },

        _getSelectedVariations: function() {
            var self = this;

            return self.variationListElements.aliasInput.filter(function(index, element) {
                // if variation is selected
                return $(element).closest('tr').find('input.ep-variationassistant-selectnewvariation').prop('checked');
            });
        },

        // step handlers
        _handleNext: function() {
            var self = this,
                o = self.options,
                attributeContainerAlias;

            if (self.currentPage === 0) {
                // abort if validation fails
                if (!self._validateAttributes()) {
                    return;
                }

                // delete empty attribute containers
                $('div' + o.attributeContainer).each(function() {
                    var $this = $(this);
                    attributeContainerAlias = $this.find('input' + o.attributeAlias);

                    // trigger click on delete icon of container, if input for AttributeAlias is empty
                    if (!attributeContainerAlias.val().length) {
                        $this.find(o.deleteAttribute).trigger('click');
                    }
                });

                self.attributeStep.addClass('HideElement');

                if (o.existingProducts.length && self._checkForUnassignedAttributes()) {
                    self._generateAttributeAssignmentTable();

                    self.defaultList.removeClass('HideElement');
                    self.backButton.removeClass('HideElement');
                    self.finishButton.addClass('HideElement');
                    self.nextButton.prop('disabled', true);
                    self.backButton.prop('disabled', false);

                    self.currentPage = 1;

                } else {
                    self._generateVariationList();

                    self.productTable.removeClass('HideElement');

                    self.backButton.removeClass('HideElement');
                    self.backButton.prop('disabled', false);
                    self.finishButton.removeClass('HideElement');
                    self.finishButton.prop('disabled', false);
                    self.nextButton.addClass('HideElement');
                    self.nextButton.prop('disabled', true);

                    self.currentPage = 2;
                }

            } else if (self.currentPage === 1 && self._validateDefaultSelection()) {
                self._generateVariationList();

                self.productTable.removeClass('HideElement');
                self.defaultList.addClass('HideElement');

                self.backButton.prop('disabled', false);
                self.finishButton.prop('disabled', false);
                self.nextButton.prop('disabled', true);

                self.backButton.removeClass('HideElement');
                self.finishButton.removeClass('HideElement');
                self.nextButton.addClass('HideElement');

                self.currentPage = 2;

            } else if (self.currentPage === 2) {
                // hide wrongly displayed button

                self.backButton.prop('disabled', false);
                self.finishButton.prop('disabled', false);
                self.nextButton.prop('disabled', true);

                self.backButton.removeClass('HideElement');
                self.finishButton.removeClass('HideElement');
                self.nextButton.addClass('HideElement');
            }
        },

        _handleBack: function() {
            var self = this;

            if (self.currentPage === 2 && self._checkForUnassignedAttributes()) {
                // show select default attribute step

                self.productTable.addClass('HideElement');

                self.defaultList.removeClass('HideElement');

                self.backButton.removeClass('HideElement');
                self.finishButton.addClass('HideElement');
                self.nextButton.removeClass('HideElement');

                self.backButton.prop('disabled', false);
                self.finishButton.prop('disabled', true);
                self.nextButton.prop('disabled', true);
                self.currentPage = 1;

            } else if (self.currentPage === 1 || self.currentPage === 2) {
                // show attribute step

                self.defaultList.addClass('HideElement');
                self.productTable.addClass('HideElement');
                self.attributeStep.removeClass('HideElement');

                self.backButton.prop('disabled', true);
                self.finishButton.prop('disabled', true);
                self.nextButton.prop('disabled', false);

                self.backButton.addClass('HideElement');
                self.finishButton.addClass('HideElement');
                self.nextButton.removeClass('HideElement');

                self.currentPage = 0;

            } else if (self.currentPage === 0) {
                // hide wrongly displayed button

                self.backButton.prop('disabled', true);
                self.finishButton.prop('disabled', true);
                self.nextButton.prop('disabled', false);

                self.backButton.addClass('HideElement');
                self.finishButton.addClass('HideElement');
                self.nextButton.removeClass('HideElement');
            }
        },

        _handleSubmit: function() {
            var self = this;

            if (self._validateNewVariations()) {
                // usage of namespace to not trigger very slow form submit handler from form.js (takes 10s+ for large number of variations)
                self.element.uiForm({showSaveWarn:false});
                self.element.trigger('submit.variationWizard');
            }
        },

        // helper functions
        _delay: function(id, cb, timeout) {
            var self = this;

            return function(evt) {
                clearTimeout(self.timeout[id]);
                self.timeout[id] = setTimeout(function() {
                    cb.bind(self)(evt);
                }, (timeout && (typeof timeout).toLowerCase() === 'number') ? timeout : 200);
            };
        },

        destroy: function () {
            var self = this;

            self.productListContainer.html('');
            self.defaultContainer.html('');

            self._superApply(arguments);
        }
    });

    return de_epages;
});
