/*globals window, define*/
/*jslint nomen:true, unparam: true*/

/*
 * @copyright       © Copyright 2006-2013, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages/taxus/inc/taxus-tools
 */

define("de_epages/taxus/inc/taxus-tools", [
    "jquery",
    "ep",

    "jquery/ui/autocomplete",
    "ep/validate",
    "ep/fn/contextorientation",
    "ep/ui/tooltip",
    "ep/fn/busy"
], function ($, ep) {
    'use strict';

    /*
     * @dictionary      ep.dict
     *
     * @translation     {Type}
     *                  {EmptyEntry}
     */

    var unity = (ep.config.unity!==undefined && ep.config.unity===true) ? true : false,

        tooltip,

        getAddressFromZip = function (options) {
            var $zip = options.zip,
                insertHandle = options.insertHandle,
                removeHandle = options.removeHandle,
                selectHandle = options.selectHandle,
                gHeight,

                split = function (val) {
                    return val.split(/,\s*/);
                },

                extractLast = function (term) {
                    return split(term).pop();
                };

            $zip
                // handle keydown and paste event
                .on("keydown", function (event) {
                    if ($zip.parent('td')) {
                        $zip.parent('td').removeClass('DialogError');
                    }
                    if (event.keyCode === $.ui.keyCode.TAB && $(this).data('uiAutocomplete').menu.activeMenu.is(':visible')) {
                        event.preventDefault();
                    }
                    if ((event.keyCode >= 48 && event.keyCode <= 57) || (event.keyCode >= 96 && event.keyCode <= 105)) {
                        insertHandle();
                    }
                })
                // handle keydown and paste event
                .on("paste", function (event) {
                    insertHandle();
                })
                // handle change event of input to decide if zip was removed
                .on("change", function (event, remove) {
                    // check if change was triggered by callback (onAfterReset) from inputaction.js
                    if (remove) {
                        removeHandle();
                    }
                })
                // handle focus event
                .on("focus", function (event) {
                    // execute onFocus callback if handed over
                    if (options.onFocus && typeof options.onFocus === 'function') {
                        options.onFocus(event, {
                            zip: $zip
                        });
                    }
                })
                .autocomplete({
                    delay: 300,
                    // triggers if input is blurred and value changed
                    change: function(event, ui) {
                        var autocompleteWidget = $zip.data('uiAutocomplete');

                        if (autocompleteWidget && autocompleteWidget.term !== $zip.val()) {
                            $zip.autocomplete('search', $zip.val());
                        }

                    },
                    create: function(event, ui) {
                        // initialize invalid error
                        $(this).data('uiAutocomplete').NOT_VALID = true;
                    },
                    response: function(event, ui) {
                        // invalid error to check the response
                        $(this).data('uiAutocomplete').NOT_VALID = !ui.content.length;
                    },
                    source: function (request, response) {
                        ep.ajax({
                            'dataType': 'json',
                            'type': 'POST',
                            'data': {
                                'ChangeAction': 'JSONViewPresenceByZipCode',
                                'ObjectID': options.taxMatrixId,
                                'ZipCode': request.term
                            }
                        })
                            .done(function (data, status, jqXHR) {
                                response(data);
                            });
                    },
                    search: function () {
                        // custom minLength
                        var term = extractLast(this.value),
                            widget = $(this).data('uiAutocomplete');

                        if (options.onBeforeSelect && typeof options.onBeforeSelect === 'function') {
                            options.onBeforeSelect($zip);
                        }

                        // invalid error to check the response
                        widget.NOT_VALID = true;

                        if (term.length < 3) {
                            // close menu (autocompletion list)
                            widget.menu.activeMenu.hide();
                            return false;
                        }

                        // selectHandle();
                    },
                    focus: function () {
                        // prevent value inserted on focus
                        return false;
                    },
                    select: function (event, ui) {
                        selectHandle(ui);

                        if (options.onAfterSelect && typeof options.onAfterSelect === 'function') {
                            options.onAfterSelect($zip);
                        }

                        // trigger change event (defined in de_epages/presentation/ui/inputaction)
                        $(this).trigger('change.presentationUiInputaction');

                        return false;
                    }
                });

            if ($zip.data('uiAutocomplete')) {
                // format data for output in autocomplete list
                $zip.data('uiAutocomplete')._renderItem = function (ul, item) {
                    return $("<li>")
                        .append("<a class='TruncateText'>" + item.ZipCode + ' - ' + item.City + ', ' + item.County + ', ' + item.State + "</a>")
                        .appendTo(ul);
                };

                // fit height of autocompletion list if content is lower as the default height
                $zip.data('uiAutocomplete')._resizeMenu = function () {
                    var menu = this.menu.element,
                        list = menu.find('li');

                    // get default height of autocompletion list when opening it the first time
                    if (!gHeight) {
                        gHeight = menu.height();
                    }

                    // fit menu width to the width of the input field
                    menu.css({
                        "min-width" : $zip.innerWidth()
                    });
                    // assign height of list items if their height is lower than default height
                    menu.height((gHeight > list.eq(0).outerHeight(true) * list.length) ? list.eq(0).outerHeight(true) * list.length : gHeight);
                };
            }
        },

        // creates a tooltip
        initTooltip = function (options) {
            var context = options.context;

            tooltip = $('<div class="ep-taxus-tools-tooltip">')
                .uiTooltip({
                    interactive: false,
                    event: 'hover',
                    context: context,
                    orientation: options.orientation,
                    oldhandling: true,
                    additionalClass: 'ep-formfield-tooltip ep-uiValidate-info',
                });
        };

    return {
        // delete physical presence
        taxusDeletePresence: function (selector, taxMatrixId) {
            $(selector).on('click', function (evt) {
                var _this = $(this),
                    form = _this.closest('form');

                ep.ajax({
                    'dataType': 'json',
                    'type': 'POST',
                    'data': {
                        'ChangeAction': 'JSONDeleteTaxAreaUS',
                        'PresenceID': _this.data('objectid'),
                        'ObjectID': taxMatrixId
                    }
                })
                    // uncomment the fail method for debug use
                    // .fail(function (jqXHR, status, error) {
                    //     console.log("fail:", error);
                    // })
                    .done(function (data, status, jqXHR) {
                        // fade out parent table row
                        _this.closest('tr').fadeOut({
                            // complete: function () {
                            //     // trigger change event to activate save button
                            //     form.trigger('change');
                            // }
                        });
                    });

            });
        },

        taxusAddPresence: function (options) {
            var $zip = options.zip,
                $hidden = {
                    // Zip2TaxCity,
                    // Zip2TaxState,
                    // Zip2TaxZipCode
                },
                addHiddenInputs = function (address) {
                    var form = $zip.closest('form');

                    $hidden.Zip2TaxCity = $('<input type="hidden" name="Zip2TaxCity" />');
                    $hidden.Zip2TaxCity.val(address.City);
                    form.append($hidden.Zip2TaxCity);

                    $hidden.Zip2TaxState = $('<input type="hidden" name="Zip2TaxState" />');
                    $hidden.Zip2TaxState.val(address.State);
                    form.append($hidden.Zip2TaxState);

                    $hidden.Zip2TaxZipCode = $('<input type="hidden" name="Zip2TaxZipCode" />');
                    $hidden.Zip2TaxZipCode.val(address.ZipCode);
                    form.append($hidden.Zip2TaxZipCode);
                },

                removeHiddenInputs = function () {
                    // remove hidden inputs from DOM
                    if ($hidden.Zip2TaxCity || $hidden.Zip2TaxState || $hidden.Zip2TaxZipCode) {
                        $.each($hidden, function (index, elem) {
                            elem.remove();
                            $hidden[index] = undefined;
                            delete $hidden[index];
                        });
                    }
                },

                opt = $.extend({
                    insertHandle: function () {
                        removeHiddenInputs();
                    },
                    removeHandle: function () {
                        removeHiddenInputs();
                    },
                    selectHandle:  function (ui) {
                        // add hidden inputs for (ZipCode, State, City)
                        addHiddenInputs(ui.item);

                        $zip[0].value = ui.item.ZipCode + ' - ' + ui.item.City + ', ' + ui.item.County + ', ' + ui.item.State;
                    },
                }, options);

            // execute additional submit handler if handed over
            if (options.onSubmit && typeof options.onSubmit === 'function') {
                $($zip[0].form).on('submit.uiForm', function(event) {
                    options.onSubmit(event, {
                        zip: $zip,
                        setHiddenInputs: removeHiddenInputs
                    });
                });
            }

            getAddressFromZip(opt);
        },

        wizardAddAddress: function (options) {
            var show = options.show ? options.show : false,
                $zip = $(options.zip.selector),
                $cityAndState = options.cityAndState,
                $city = options.spanCity,
                $state = options.spanState,
                $hidden = {
                    city: $(options.city.selector),
                    state: $(options.state.selector)
                },
                setHiddenInputs = function (address) {
                    var city = address ? address.City : '',
                        state = address ? address.State : '';

                    // Set hidden inputs
                    $hidden.city.val(city);
                    $hidden.state.val(state);

                    // Set correct text
                    $city.text(city);
                    $state.text(state);

                    if (unity) {
                        // show/hide city-state-section
                        $cityAndState[address ? 'show' : 'hide'](0);
                    } else {
                        // set class for city-state-section
                        $cityAndState[address ? 'removeClass' : 'addClass']('TextWeaker');
                    }
                },
                opt = $.extend(options, {
                    insertHandle: function () {
                        setHiddenInputs();
                    },
                    removeHandle: function () {
                        setHiddenInputs();
                    },
                    selectHandle:  function (ui) {
                        // set hidden inputs for State and City
                        setHiddenInputs(ui.item);

                        $zip[0].value = ui.item.ZipCode;
                    },
                    zip: $zip
                });


            if (unity && !show) {
                setHiddenInputs();
            }

            // execute additional submit handler if handed over
            if (options.onSubmit && typeof options.onSubmit === 'function') {
                $($zip[0].form).on('submit.uiForm', function(event) {
                    options.onSubmit(event, {
                        zip: $zip,
                        setHiddenInputs: setHiddenInputs
                    });
                });
            }

            getAddressFromZip(opt);
        },

        // tooltips
        tooltip: function (options) {
            var context = options.context,
                orientation = options.orientation ? options.orientation : 'bottom right';

            // create a tooltip if necessary
            if (tooltip === undefined) {
                initTooltip({
                    context: context,
                    orientation: orientation
                });
            }

            // update tooltip options
            tooltip.uiTooltip('option', 'context', context);
            tooltip.html(options.message);
            tooltip.uiTooltip('show');

            options.interactiveElement.one('click', function () {
                tooltip.uiTooltip('hide');
            });
        }

    };

});

