/*jslint browser: true, nomen: true, plusplus: true, regexp: true*/
/*global define*/
define('testsuite/qunit-helper', ['jquery'], function ($) {

    'use strict';

    var constructor = function (ObjectType) {
            var ret = function () {
                    return;
                };

            ret.toString = function () {
                return '[object ' + ObjectType + 'Constructor]';
            };

            return ret;
        },

        types = {

            File: function (name, type, size) {
                return {
                    constructor: constructor('File'),
                    fileName: name || "example.jpg",
                    name: name || "example.jpg",
                    fileType: type || "image/jpeg",
                    type: type || "image/jpeg",
                    size: size || 1024
                };
            },

            HTMLInputElement: function (extend) {
                return $.extend({
                    constructor: constructor('HTMLInputElement'),
                    nodeType: 1,
                    nodeName: 'INPUT',
                    getElementById: function () {
                        return null;
                    },
                    getElementsByName: function () {
                        return [];
                    },
                    getElementsByTagName: function () {
                        return [];
                    },
                    getElementsByClassName: function () {
                        return [];
                    }
                }, extend);
            }

        },

        // prepare toString check for fake objects
        _toString = Object.prototype.toString;

    function fake() {
        var type = Array.prototype.shift.call(arguments);

        if (!fake[type]) {
            if (types[type]) {
                fake[type] = types[type];
            } else {
                return;
            }
        }

        return fake[type].apply(fake[type], arguments);
    }

    Object.prototype.toString = function () {
        var type,
            objectString;

        if (this.constructor) {
            this.constructor.toString().replace(/^(\[object )(.*?)(Constructor)(\])$/, function (all, $1, $2, $3, $4) {
                /*jslint unparam: true*/
                type = $2;
                objectString = $1 + $2 + $4;
                return all;
            });
        }

        return fake[type] ? objectString : _toString.call(this);
    };

    // Helper to series function calls
    // Usage: series(timebox, function, function, ...);
    function asyncTimeboxSeries() {
        var timebox = Array.prototype.shift.apply(arguments, []),
            args = arguments,
            progress = 0;

        function createCallback(i) {
            return function () {
                var callback;

                if (i === progress && typeof args[i] === 'function') {
                    progress++;

                    callback = createCallback(i + 1);

                    // Set max time for tests
                    setTimeout(function () {
                        if (progress < i + 2) {
                            callback();
                        }
                    }, timebox);

                    // Run next item in series
                    args[i](callback);
                }
            };
        }

        createCallback(0)();
    }

    return {
        fake: fake,
        asyncTimeboxSeries: asyncTimeboxSeries
    };

});